import { Http } from '@zeedhi/core';
import { GenericValueController } from '@/controllers/GenericValueController';
import { Utilities } from '@/util/Utilities';
import SharedInfo from '@/helper/SharedInfo';

export class CardsController {
  public DIASEST: string = '';

  public VRDESVIO: string = '';

  public VRSOLEXTRA: string = '';

  public PROCESSINGDATE: string = '';

  public PROXLIBCARD: string = '';
  
  public VRAJUSTE: string = '';
  
  public DIAAPROV: string = '';
  
  public IMPACTO: string = '';
  
  public QTDE: string = '';
  
  public QTDIASATRASO: string = '';
  
  public getDataFillCards(components) {
    const childrenComponents = components.component.parent.children;
    const componentForProps = childrenComponents.find((child) => child.name && child.name === components.component.name);
    const props = componentForProps.props;
    const routeName = props.routeName;
    const columnName = props.columnName;
    const type = props.type;
    const sendObject = {
      filter: [],
    };
    const filter = props.filter;
    const genericValues = new GenericValueController();
    const formatting = new Utilities();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (filter) {
      sendObject.filter = formatting.formatFilter(filter);
    }
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        /* eslint no-console: ["error", { allow: ["warn", "error", "info", "log"] }] */
        const data = response.data.dataset[routeName];
        const [firstRecord] = Array.isArray(data) ? data : [];
        
        this[columnName] = formatting.formatValue(type, firstRecord.VALUE);
        this.defineColor(components.component, firstRecord.VALUE);
      });
    } else {
      const object = genericValues.valueList.find((obj) => obj.columnName === columnName);
      
      if (object) {
        this[columnName] = formatting.formatValue(type, object.value);
        this.defineColor(components.component, object.value);
      } else {
        this[columnName] = 0;
        this.defineColor(components.component, 0);
      }
    }
  }
  
  public defineColor(component, value) {
    const indic = SharedInfo.getInfoIndic();
    const zdTag = component.parent.children.find((obj) => (obj.component === 'ZdTag'));
    const zdCard = component.parent;
    const lighten = new Utilities();
    const percentReduction = 18;
    let zdHeader;

    if (zdTag) {
      zdHeader = zdTag.children.find((obj) => (obj.component === 'ZdHeader'));
    } else {
      zdHeader = component.parent.children.find((obj) => (obj.component === 'ZdHeader'));
    }
    let info;

    switch (zdHeader.name) {
      case 'daysOfStockHeader':
        info = indic.find((obj) => obj.id === 1);
        zdHeader.color = this.conditionColor(value, info.limMinimo, info.limMeta, 'N');
        break;
      case 'daysOfDelayHeader':
        info = indic.find((obj) => obj.id === 3);
        zdHeader.color = this.conditionColor(value, info.limMinimo, info.limMeta, 'Y');
        break;
      case 'adjustmentValueHeader':
        info = indic.find((obj) => obj.id === 4);
        zdHeader.color = this.conditionColor(value, info.limMinimo, info.limMeta, 'Y');
        break;
      case 'extraRequestHeader':
        info = indic.find((obj) => obj.id === 5);
        zdHeader.color = this.conditionColor(value, info.limMinimo, info.limMeta, 'Y');
        break;
      case 'consumptionDiversionHeader':
        info = indic.find((obj) => obj.id === 19);
        zdHeader.color = this.conditionColor(value, info.limMinimo, info.limMeta, 'Y');
        break;
      case 'daysMenuApprovalTextHeader':
        info = indic.find((obj) => obj.id === 21);
        zdHeader.color = this.conditionColor(value, info.limMinimo, info.limMeta, 'N');
        break;
      case 'pendingReleasesHeader': zdHeader.color = '#0B5394';
        break;
      case 'processingDateHeader': zdHeader.color = '#0B5394';
        break;
      default: component.color = '#F3F6F4';
    }
    zdCard.color = lighten.lightenColor(zdHeader.color, percentReduction);
  }
  
  conditionColor(value: number, minimum: number, goal: number, biggerToSmaller: string): string {
    let color;
    if (biggerToSmaller === 'N') {
      color = '#CC0000';
      if (value > goal) {
        color = '#0B5394';
      } else if (value > minimum) {
        color = '#F1C232';
      }
    } else {
      color = '#F1C232';
      if (value < minimum) {
        color = '#6AA84F';
      } else if (value > goal) {
        color = '#CC0000';
      }
    }
    return color;
  }
  
  public tableData({ component }) {
    const routeName = component.name;
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    const formatting = new Utilities();
    const sendObject = {
      filter: [],
    };
    const filter = formatting.formatFilter(component.datasource.filter);
    
    if (filter) {
      sendObject.filter = filter;
    }
    let data;
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        component.datasource.setFilter([]);
        data = response.data.dataset[routeName].map((obj) => ({
          ...obj,
          VALUE: formatting.formatValue('float', obj.VALUE),
        }));
        component.datasource.updateData(data);
      });
    } else {
      data = genericValues.valueGridCard[component.name].map((obj) => ({
        ...obj,
        VALUE: formatting.formatValue('float', obj.VALUE),
      }));
      component.datasource.setFilter([]);
      component.datasource.updateData(data);
    }
  }
}
