import { Http, Metadata } from '@zeedhi/core';
import { Indicators } from '@/helper/Indicators';
import SharedInfo from '@/helper/SharedInfo';
import { DashboardGrid } from '@zeedhi/zd-dashboard-grid';
import { ZdGrid, ZdTabs, ZdButtonGroup } from '@zeedhi/vuetify';
import { Utilities } from '@/util/Utilities';
import { GenericValueController } from '@/controllers/GenericValueController';
import ChangeLabel from '@/helper/Labels';

export class DashboardController {
  private _accessIndicators: number[];

  public editingModeDash = false;
  
  public secondLoad = 0;
  
  public homePanel = null;
 
  public get accessIndicators(): number[] {
    /* eslint no-underscore-dangle: 0 */
    return this._accessIndicators;
  }

  public set accessIndicators(accessIndicators: number[]) {
    /* eslint no-underscore-dangle: 0 */
    this._accessIndicators = accessIndicators;
  }
  
  public getDataGrid({ component }) {
    const formatting = new Utilities();
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    const routeName = component.children[0].datasourceName;
    const sendObject = {
      filter: [],
    };
    let dayOfWeekProc;
    let data;
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        /* eslint no-console: ["error", { allow: ["warn", "error", "info", "log"] }] */
        data = response.data.dataset[routeName].map((obj) => {
          Object.keys(obj).forEach((key) => {
            if (typeof obj[key] === 'number') {
              if (key === 'NRDTPROC') {
                dayOfWeekProc = obj[key];
              } else if (key !== 'NRDIA') {
                obj[key] = formatting.formatValue('float', obj[key]);
              }
            } else if (obj[key] instanceof Date) {
              console.log(obj[key]);
            } else if (key === 'NMITEM') {
              obj[key] = ChangeLabel.getLabel(obj[key]);
            }
            if (key === 'NRDTPROC') {
              dayOfWeekProc = obj[key];
            }
          });
        
          return obj;
        });
        
        component.datasource.updateData(data);
        if (routeName === 'todayDeliveries') {
          const zdgroupbutton = Metadata.getInstance<ZdButtonGroup>('daysWeek');
          
          zdgroupbutton.selectedButtons = dayOfWeekProc - 1;
          component.datasource.setFilter({
            NRDIA: dayOfWeekProc,
          });
        }
      });
    } else {
      data = genericValues.valueGrid[component.name].map((obj) => {
        Object.keys(obj).forEach((key) => {
          if (typeof obj[key] === 'number') {
            if (key === 'NRDTPROC') {
              dayOfWeekProc = obj[key];
            } else if (key !== 'NRDIA') {
              obj[key] = formatting.formatValue('float', obj[key]);
            }
          } else if (obj[key] instanceof Date) {
            console.log(obj[key]);
          } else if (key === 'NMITEM') {
            obj[key] = ChangeLabel.getLabel(obj[key]);
          }
        });
      
        return obj;
      });
      
      component.datasource.updateData(data);
      if (routeName === 'todayDeliveries') {
        const zdgroupbutton = Metadata.getInstance<ZdButtonGroup>('daysWeek');
        
        zdgroupbutton.selectedButtons = dayOfWeekProc - 1;
        component.datasource.setFilter({
          NRDIA: dayOfWeekProc,
        });
      }
    }
  }
  
  public async cardList({ component }) {
    component.cards = [];
    // await this.accessibleIndicators();
    await SharedInfo.waitMyBrother();
    const indic = new Indicators();
    const visible = this.accessIndicators;
    let visibleCards = [];
    try {
      switch (component.name) {
        case 'dashGU': visibleCards = this.sortCards(indic.cardListGU.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          break;
        case 'dashGO': visibleCards = this.sortCards(indic.cardListGO.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          break;
        case 'dashTD': visibleCards = this.sortCards(indic.cardListTD.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          break;
        case 'dashTA': visibleCards = this.sortCards(indic.cardListTA.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          break;
        default: visibleCards = [];
      }
      
      component.cards = visibleCards;
    } catch (error) {
      // Adicionar mensagem de erro
      component.cards = [];
    }
  }
  
  public async accessibleIndicators() {
    const indic = new Indicators();
    const sendObject = {
      filter: [],
    };
    
    try {
      if (typeof this.accessIndicators === 'undefined') {
        const routeName = 'visibleIndicators';
        const routePanel = 'homePanel';
        const response = await Http.post('/visibleIndicators', sendObject);
        this.accessIndicators = response.data.dataset[routeName].concat(indic.auxDash);
        this.homePanel = response.data.dataset[routePanel][0].HOMEPANEL;
      }
    } catch (error) {
      // Adicionar mensagem de erro
    }
  }
  
  public async visiblePanel({ component }) {
    await this.accessibleIndicators();
    const indic = new Indicators();
    
    let panelVisible = false;
    let dashVisible = false;
    
    component.tabs.forEach((obj) => {
      switch (obj.name) {
        case 'unitManager': panelVisible = indic.cardsGU.some((item) => this.accessIndicators.includes(item));
          break;
        case 'operationalManager': panelVisible = indic.cardsGO.some((item) => this.accessIndicators.includes(item));
          break;
        case 'costAnalysis': panelVisible = indic.cardsTD.some((item) => this.accessIndicators.includes(item));
          break;
        case 'tracking': panelVisible = false;
          break;
        default: panelVisible = false;
      }
      obj.isVisible = panelVisible;
      dashVisible = dashVisible || panelVisible;
    });
    let activePanel = localStorage.getItem('ACTIVE_PANEL');
    component.activeTabValue = activePanel !== 'null' ? activePanel : this.homePanel;
    activePanel = null;
    component.isVisible = dashVisible;
    this.secondLoad = 1;
  }
  
  public cancelEditOrder() {
    this.editingModeDash = false;
    window.location.reload();
  }
  
  public confirmEditOrder(components) {
    let panel = [];
    switch (components.component.name) {
      case 'confirmEditOrderGU': panel = ['dashGU', 0];
        break;
      case 'confirmEditOrderGO': panel = ['dashGO', 1];
        break;
      case 'confirmEditOrderAC': panel = ['dashTD', 2];
        break;
      case 'confirmEditOrderTA': panel = ['dashTA', 3];
        break;
      default: panel = ['', ''];
    }
    if (panel[0] !== '') {
      const dash = Metadata.getInstance<DashboardGrid>(panel[0]);
      /* const newOrder = dash.cards.map((obj) => `${obj.i}|${obj.h}|${obj.w}|${obj.x}|${obj.y}`).join(','); */
      const newOrder = dash.cards.map((obj) => `${obj.i}|${obj.x}|${obj.y}`).join(',');
      const sendObject = {
        orderIndicDash: newOrder,
        selectedPanel: panel[0],
      };
  
      Http.post('/changeOrderDash', sendObject).then(() => {
        localStorage.setItem('ACTIVE_PANEL', panel[1]);
        this.enabledTabsDashboard(false);
      });
    }
    this.editingModeDash = false;
  }
  
  sortCards(cards: any, panel: string): any {
    const cardOrder = SharedInfo.getOrderPanel(panel);
    let cardsTmp;
    
    if (cardOrder) {
      const savedCards = cardOrder.split(',').map((item) => {
        const [i, x, y] = item.split('|').map(parseFloat);
        return {
          i, x, y,
        };
      });
      
      cardsTmp = cards.map((obj) => {
        const auxCardObj = savedCards.find((savedCardObj) => savedCardObj.i === obj.i);
      
        if (auxCardObj) {
          /* obj.h = auxCardObj.h; */
          /* obj.w = auxCardObj.w; */
          obj.x = auxCardObj.x;
          obj.y = auxCardObj.y;
        }

        return obj;
      });
    } else {
      cardsTmp = cards;
    }
    return cardsTmp;
  }

  public handlingCards({ component }) {
    if (component.cards.length !== 0) {
      if (this.secondLoad === 2) {
        this.editingModeDash = true;
        this.enabledTabsDashboard(true);
      } else {
        this.secondLoad += 1;
      }
    }
  }
  
  public enabledTabsDashboard(editModeTab) {
    const tabs = Metadata.getInstance<ZdTabs>('dashboardNext');
    const activeTab = tabs.activeTab;
    tabs.tabs.forEach((i) => {
      if (tabs.tabs[activeTab].name !== i.name) {
        i.disabled = editModeTab;
      }
    });
  }
  
  get editingMode() {
    return this.editingModeDash;
  }
  
  public showDescription() {
    
  }
  
  public async panelConfig() {
    if (SharedInfo.getSingleRun) {
      await this.accessibleIndicators();
      const sendObject = {
        filter: [],
      };

      try {
        const response = await Http.post('/panelConfig', sendObject);
        const panelConfig = response.data.dataset.panelConfig[0];
        const indicConfig = response.data.dataset.indicConfig;
      
        indicConfig.forEach((obj) => {
          SharedInfo.setLimit(obj.NRINDICADORTF, obj.VRLIMMETA, obj.VRLIMMINIMO);
        });
        SharedInfo.setOrderPanel(
          panelConfig.loaded,
          panelConfig.DASHGU,
          panelConfig.DASHGO,
          panelConfig.DASHTD,
          panelConfig.DASHTA,
        );
      
        if (SharedInfo.loadingSolve) {
          SharedInfo.loadingSolve();
        }
      } catch (error) {
        console.log('teste');
      }
    }
    this.editingModeDash = false;
    this.enabledTabsDashboard(false);
  }
  
  public daysWeek(components) {
    const todayDeliveriesGrid = Metadata.getInstance<ZdGrid>('todayDeliveriesGrid');
    let nrDay;
    
    switch (components.component.name) {
      case 'DOM': nrDay = 1;
        break;
      case 'SEG': nrDay = 2;
        break;
      case 'TER': nrDay = 3;
        break;
      case 'QUA': nrDay = 4;
        break;
      case 'QUI': nrDay = 5;
        break;
      case 'SEX': nrDay = 6;
        break;
      default: nrDay = 7;
    }
    
    todayDeliveriesGrid.datasource.setFilter({
      NRDIA: nrDay,
    });
  }

  public loadingDash() {
    this.secondLoad = 0;
  }
}
